<?php
//FOR DEBUG ONLY

use PhpOffice\PhpSpreadsheet\IOFactory;

// ini_set('display_errors', 1);
// ini_set('display_startup_errors', 1);
// error_reporting(E_ALL);


/**
 * Plugin Name:       JTB Analitics
 * Description:       Dipsaly analitics for dinamic pages  
 * Version:           1.0.1
 * Author:            JTB Studios
 * Author URI:        https://jtbstudios.com.au
 * Text Domain:       jtbstudios
 */


//plugin source
if (!defined('JTB_ANALITICS_URL'))
    define('JTB_ANALITICS_URL', plugin_dir_url(__FILE__));
//plugin path
define('JTB_ANALITICS_PLUGIN', __FILE__);
define('JTB_ANALITICS_DIR', untrailingslashit(dirname(JTB_ANALITICS_PLUGIN)));

//DEACTIVATION Plugin
register_deactivation_hook(__FILE__, 'jtbAnalitics_deactivation_plugin');
function jtbAnalitics_deactivation_plugin()
{
}

/*
 * Plugin constants
 */


/*
 * Main class
 */
class JtbAnalitics
{
    private $_jtbAnaliticsAjax = 'jtb_analitics_admin';

    /**
     * constructor.
     *
     * The main plugin actions registered for WordPress
     */
    public function __construct()
    {

        //Admin page
        add_action('admin_menu', array($this, 'addAdminMenu'));
        //POST TYPE
        add_action('init', array($this, 'AnaliticsPostType'));
        //ACF FIELD
        add_action('init', array($this, 'SetAcfFiled'));
        //FRON PAGE SCRIPT
        add_action('init', array($this, 'AddFrontPageScript'));


        if (is_admin()) {
            //add admin scripts
            add_action('admin_enqueue_scripts', array($this, 'addAdminScripts'));
        }


        add_action('wp_ajax_nopriv_jtbCountEvent', array($this, 'jtbCountEvent'));
        add_action('wp_ajax_jtbCountEvent', array($this, 'jtbCountEvent'));
    }

    /**
     * Set Admin menu
     */
    public function addAdminMenu()
    {
        add_menu_page(
            __('Jtb Analitics', 'epicupImporter'),
            __('Jtb Analitics', 'epicupImporter'),
            'manage_options',
            'epicupImporter',
            array($this, 'adminLayout'),
            ''
        );
    }

    /**
     * Adds Admin Scripts for the Ajax call
     */
    public function addAdminScripts()
    {
        wp_enqueue_script('jtbAnaliticsMainScript', JTB_ANALITICS_URL . '/js/adminMain.js', array(), 1.0);

        $admin_options = array(
            'ajax_url' => admin_url('admin-ajax.php'),
            '_nonce' => wp_create_nonce($this->_jtbAnaliticsAjax),
        );

        wp_localize_script('jtbAnaliticsMainScript', 'jtbAnalitics', $admin_options);
    }

    /**
     * FRONT PAGE SCRIPTS
     */
    public function AddFrontPageScript()
    {
        wp_register_script('frotPageScript', JTB_ANALITICS_URL . '/js/frotPageScript.js', array(), '1.0.0', true);
        wp_enqueue_script('frotPageScript');

        $front_options = array(
            'ajax_url' => admin_url('admin-ajax.php'),
            '_nonce' => wp_create_nonce($this->_jtbAnaliticsAjax),
        );

        wp_localize_script('frotPageScript', 'jtbAnalitics', $front_options);
    }

    /**
     * MAIN ADMIN PAGE LAYOUT
     * 
     */
    public function adminLayout()
    {

        $this->CreateCsvFile();

        $args = array(
            'posts_per_page'   => -1, // -1 here will return all posts
            'post_type'        => 'jtbanalitics', //your custom post type
            'post_status'      => 'publish',
        );
        $posts = get_posts($args);
        $fileUrl = JTB_ANALITICS_URL.'csv_files/analitics_data.csv';

?>
        <div class="wrap">
            <h1>JTB Analitics </h1>
            <div class="postbox">
                <div class="wrap">
                    <h2 class="title"><span>Pages analitics</span> <a href="<?php echo $fileUrl; ?>" download> Download CSV file </a></h2>
                </div>
                <div class="inside">
                    <table class="form-table">
                        <thead>
                            <tr>
                                <th scope="row">Page</th>
                                <th scope="row">Content Likes</th>
                                <th scope="row">Content Views</th>
                                <th scope="row">Content Shares</th>
                                <th scope="row">Content Downloads</th>
                                <th scope="row">Driven to Act Survey</th>
                            </tr>
                        </thead>
                        <tbody>
                            <?php
                            foreach ($posts as $post) {
                                $likes = get_field('content_likes', $post->ID);
                                $views = get_field('content_views', $post->ID);
                                $shares = get_field('content_shares', $post->ID);
                                $download = get_field('content_downloads', $post->ID);
                                $survey = 0; //get_field('driven_to_act_survey', $post->ID);

                                $likes = $likes != null ? $likes : 0;
                                $views = $views != null ? $views : 0;
                                $shares = $shares != null ? $shares : 0;
                                $download = $download != null ? $download : 0;

                                echo '<tr>';
                                echo '<td>' . $post->post_title . '</td>';
                                echo '<td>' . $likes . '</td>';
                                echo '<td>' . $views . '</td>';
                                echo '<td>' . $shares . '</td>';
                                echo '<td>' . $download . '</td>';
                                echo '<td>' . $survey . '</td>';
                            }
                            echo '</tr>';
                            ?>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
<?php
    }

    /**
     * ADD NEW POST TYPE
     */

    public function AnaliticsPostType()
    {
        $labels = array(
            'name'                => _x('JtbAnalitcs', 'Post Type General Name', 'text_domain'),
            'singular_name'       => _x('JtbAnalitcs', 'Post Type Singular Name', 'text_domain'),
            'menu_name'           => __('JtbAnalitcs', 'text_domain'),
            'name_admin_bar'      => __('JtbAnalitcs', 'text_domain'),
            'parent_item_colon'   => __('JtbAnalitcs:', 'text_domain'),
            'all_items'           => __('All', 'text_domain'),
            'add_new_item'        => __('New', 'text_domain'),
            'add_new'             => __('New', 'text_domain'),
            'new_item'            => __('New', 'text_domain'),
            'edit_item'           => __('Edit', 'text_domain'),
            'update_item'         => __('Update', 'text_domain'),
            'view_item'           => __('View', 'text_domain'),
            'search_items'        => __('Search', 'text_domain'),
            'not_found'           => __('Not found', 'text_domain'),
            'not_found_in_trash'  => __('Not found in Trash', 'text_domain'),
        );
        $args = array(
            'label'               => __('JtbAnalitcs', 'text_domain'),
            'description'         => __('JtbAnalitcs ', 'text_domain'),
            'labels'              => $labels,
            'supports'            => array('title', 'editor'),
            'taxonomies'          => array(''),
            'hierarchical'        => true,
            'public'              => true,
            'show_ui'             => true,
            'show_in_menu'        => true,
            'menu_position'       => 5,
            'show_in_admin_bar'   => true,
            'show_in_nav_menus'   => true,
            'can_export'          => true,
            'has_archive'         => true,
            'exclude_from_search' => false,
            'publicly_queryable'  => true,
            'capability_type'     => 'page',
            'menu_icon'           => 'dashicons-chart-line',
            // 'rewrite' => array(
            //     'slug' => 'banking'
            // )
        );

        register_post_type('jtbanalitics', $args);
    }


    /**
     * ACF
     */
    public function SetAcfFiled()
    {
        if (function_exists('acf_add_local_field_group')) {

            acf_add_local_field_group(array(
                'key' => 'group_606c1c98b8634',
                'title' => 'JTB Annalitics ACF',
                'fields' => array(
                    array(
                        'key' => 'field_606c1cec65f3f',
                        'label' => 'Content Likes',
                        'name' => 'content_likes',
                        'type' => 'number',
                        'instructions' => '',
                        'required' => 0,
                        'conditional_logic' => 0,
                        'wrapper' => array(
                            'width' => '',
                            'class' => '',
                            'id' => '',
                        ),
                        'default_value' => '',
                        'placeholder' => '',
                        'prepend' => '',
                        'append' => '',
                        'min' => '',
                        'max' => '',
                        'step' => '',
                    ),
                    array(
                        'key' => 'field_606c1d0a65f40',
                        'label' => 'Content Views',
                        'name' => 'content_views',
                        'type' => 'number',
                        'instructions' => '',
                        'required' => 0,
                        'conditional_logic' => 0,
                        'wrapper' => array(
                            'width' => '',
                            'class' => '',
                            'id' => '',
                        ),
                        'default_value' => '',
                        'placeholder' => '',
                        'prepend' => '',
                        'append' => '',
                        'min' => '',
                        'max' => '',
                        'step' => '',
                    ),
                    array(
                        'key' => 'field_606c1d1e65f41',
                        'label' => 'Content Shares',
                        'name' => 'content_shares',
                        'type' => 'number',
                        'instructions' => '',
                        'required' => 0,
                        'conditional_logic' => 0,
                        'wrapper' => array(
                            'width' => '',
                            'class' => '',
                            'id' => '',
                        ),
                        'default_value' => '',
                        'placeholder' => '',
                        'prepend' => '',
                        'append' => '',
                        'min' => '',
                        'max' => '',
                        'step' => '',
                    ),
                    array(
                        'key' => 'field_606c1d3065f42',
                        'label' => 'Content Downloads',
                        'name' => 'content_downloads',
                        'type' => 'number',
                        'instructions' => '',
                        'required' => 0,
                        'conditional_logic' => 0,
                        'wrapper' => array(
                            'width' => '',
                            'class' => '',
                            'id' => '',
                        ),
                        'default_value' => '',
                        'placeholder' => '',
                        'prepend' => '',
                        'append' => '',
                        'min' => '',
                        'max' => '',
                        'step' => '',
                    ),
                    array(
                        'key' => 'field_606c1d3d65f43',
                        'label' => 'Driven to Act Survey',
                        'name' => 'driven_to_act_survey',
                        'type' => 'number',
                        'instructions' => '',
                        'required' => 0,
                        'conditional_logic' => 0,
                        'wrapper' => array(
                            'width' => '',
                            'class' => '',
                            'id' => '',
                        ),
                        'default_value' => '',
                        'placeholder' => '',
                        'prepend' => '',
                        'append' => '',
                        'min' => '',
                        'max' => '',
                        'step' => '',
                    ),
                ),
                'location' => array(
                    array(
                        array(
                            'param' => 'post_type',
                            'operator' => '==',
                            'value' => 'jtbanalitics',
                        ),
                    ),
                ),
                'menu_order' => 0,
                'position' => 'normal',
                'style' => 'default',
                'label_placement' => 'top',
                'instruction_placement' => 'label',
                'hide_on_screen' => '',
                'active' => true,
                'description' => '',
            ));
        }
    }

    /**
     * SET NUMBER OF EVENTS
     */

    public function jtbCountEvent()
    {
        
        header("Access-Control-Allow-Origin: *");
        // header('Content-Type: application/json');

        ini_set('display_errors', 1);
        ini_set('display_startup_errors', 1);
        error_reporting(E_ALL);

        $page = get_page_by_title('HOME PAGE', OBJECT, 'jtbanalitics');

        $eventName = $_POST['eventName'];
        $pageName = $_POST['pageName'];

        $pageName = str_replace('\"','"',$pageName);
        $pageName = str_replace("\'","'",$pageName);
        //$the_slug = strtolower(str_replace(' ','-',$pageName));


        var_dump($pageName);

        // $args = array(
        // 'name'        => $the_slug,
        // 'post_type'   => 'post',
        // 'post_status' => 'publish',
        // 'numberposts' => 1
        // );
        // $my_posts = get_posts($args);

        $page = get_page_by_title(html_entity_decode($pageName), OBJECT, 'jtbanalitics');

         var_dump($page);
         if ($page == null) {
             $this->CreateNewPost($pageName, $eventName);
         } else {
             $this->UpdateEventCount($page->ID, $eventName);
         }

        die;
    }

    private function CreateNewPost($pageName, $eventName)
    {
        global $user_ID;

        $post = array(
            'post_title' => $pageName,
            'post_status' => 'publish',
            'post_date' => date('Y-m-d H:i:s'),
            'post_type' => 'jtbanalitics',
        );

        $post_id = wp_insert_post($post);
        $this->UpdateEventCount($post_id, $eventName);
    }

    private function UpdateEventCount($postId, $eventName)
    {
        $likes = get_field('content_likes', $postId);
        $views = get_field('content_views', $postId);
        $shares = get_field('content_shares', $postId);
        $download = get_field('content_downloads', $postId);
        $survey = 0; //get_field('driven_to_act_survey', $post->ID); //TODO add this 

        if ($eventName == 'LIKE_CLICK') {
            $likes  = $likes != null ? $likes + 1 : 1;
        } else if ($eventName == 'SHARE_CLICK') {
            $shares = $shares != null ? $shares + 1 : 1;
        } else if ($eventName == 'DOWNLOAD_CLICK') {
            $download = $download != null ? $download + 1 : 1;
        } else if ($eventName == 'ON_VIEW') {
            $views = $views + 1;
        }

        update_field('content_likes', $likes, $postId);
        update_field('content_views', $views, $postId);
        update_field('content_shares', $shares, $postId);
        update_field('content_downloads', $download, $postId);
        //update_field('driven_to_act_survey',$variable,$postId); //TODO add this 

    }

    private function CreateCsvFile()
    {
        $args = array(
            'posts_per_page'   => -1, // -1 here will return all posts
            'post_type'        => 'jtbanalitics', //your custom post type
            'post_status'      => 'publish',
        );
        $posts = get_posts($args);

        $csvData = array();

        $csvHeader = array('Page', 'Content Likes', 'Content Views', 'Content Shares', 'Content Downloads', 'Driven to Act Survey');

        array_push($csvData, $csvHeader);

        foreach ($posts as $post) {
            $likes = get_field('content_likes', $post->ID);
            $views = get_field('content_views', $post->ID);
            $shares = get_field('content_shares', $post->ID);
            $download = get_field('content_downloads', $post->ID);
            $survey = 0; //get_field('driven_to_act_survey', $post->ID);

            $likes = $likes != null ? $likes : 0;
            $views = $views != null ? $views : 0;
            $shares = $shares != null ? $shares : 0;
            $download = $download != null ? $download : 0;

            $data = array($post->post_title, $likes, $views, $shares, $download, $survey);
            array_push($csvData, $data);
        }

        $filePath = JTB_ANALITICS_DIR . '/csv_files/analitics_data.csv';

        $fp = fopen($filePath, 'w');
        foreach ($csvData as $field) {
            fputcsv($fp, $field);
        }

        fclose($fp);

        

    }
}


$epicupImporter = new JtbAnalitics();
