<?php


/**
 * The public-facing functionality of the plugin.
 *
 *
 */

namespace WidgetForEventbriteAPI\FrontEnd;

use ActionScheduler_Store;
use stdClass;
use WidgetForEventbriteAPI\Includes\ICS;
use WidgetForEventbriteAPI\Includes\Template_Loader;
use WidgetForEventbriteAPI\Includes\Eventbrite_Query;


class FrontEnd {

	/**
	 * The ID of this plugin.
	 *
	 */
	private $plugin_name;
	/**
	 * The version of this plugin.
	 *
	 */
	private $version;

	private $utilities;
	/**
	 * @var \Freemius $freemius Object for freemius.
	 */

	private $freemius;

	/**
	 * Initialize the class and set its properties.
	 *
	 */
	public function __construct( $plugin_name, $version, $utilities = null ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		$this->utilities   = $utilities;
		global $wfea_fs;
		$this->freemius = $wfea_fs;

	}

	public static function default_args() {
		/**
		 * @var \Freemius $wfea_fs Object for freemius.
		 */
		global $wfea_fs;
		$defaults = array(
			'booknow'        => 'true',
			'booknow_text'   => esc_html__( 'Register »', 'widget-for-eventbrite-api' ),
			'cssid'          => '',
			'css_class'      => '',
			'date'           => 'true',
			'debug'          => 'false',
			'excerpt'        => 'true',
			'layout'         => 'widget',
			'length'         => 50,
			'limit'          => 5,
			'newtab'         => 'false',
			'order_by'       => '',
			'readmore'       => 'true',
			'readmore_text'  => esc_html__( 'Read More »', 'widget-for-eventbrite-api' ),
			'status'         => 'live',
			'thumb'          => 'true',
			'thumb_align'    => 'eaw-aligncenter',
			'thumb_default'  => 'https://dummyimage.com/600x400/f0f0f0/ccc',
			'thumb_original' => 'false',
			'thumb_width'    => 300,
			'tickets'        => 'false',
			'widgetwrap'     => 'true',
		);
		if ( $wfea_fs->can_use_premium_code__premium_only() ) {
			$defaults_silver = array(
				'category_id'                => '',
				'content'                    => 'false',
				'deduplicate'                => 'true',
				'display_private'            => 'true',
				'eb_id'                      => '',
				'filter_location'            => '',
				'filter_title'               => '',
				'first_of_series'            => 'false',
				'format_id'                  => '',
				'layout'                     => '1',
				'location'                   => '',
				'location_title'             => '',
				'long_description'           => 'false',
				'no_events_found_text'       => esc_html__( 'No Events Found', 'widget-for-eventbrite-api' ),
				'online_events_address_text' => '',
				'online_events_only'         => 'false',
				'organization_id'            => '',
				'organizer_id'               => '',
				'search'                     => '',
				'show_date_time'             => 'true',
				'show_end_time'              => 'true',
				'start_date_range_end'       => '',
				'start_date_range_start'     => '',
				'style'                      => 'default',
				'subcategory_id'             => '',
				'venue_id'                   => '',
				'venue_name'                 => 'false',

			);
			$defaults        = array_merge( $defaults, $defaults_silver );
			if ( $wfea_fs->is_plan_or_trial( 'gold' ) ) {
				$defaults_gold = array(
					'api_key'                          => '',
					'cal_default_view'                 => '',
					'cal_list_days'                    => 0,
					'cal_list_header_center'           => '',
					'cal_list_header_left'             => 'title',
					'cal_list_header_right'            => 'prev,next',
					'canceled_event_button'            => '',
					'coming_soon_button'               => '',
					'direct_to_external'               => 'true',
					'link_custom_page'                 => '',
					'music_show_age_restriction'       => 'false',
					'music_show_age_restriction_label' => esc_html__( 'Min. Age: ', 'widget-for-eventbrite-api' ),
					/* translators: %1$s time (e.g.Show starts: 19:30) */
					'music_show_door_no_time_label'    => esc_html__( 'Show: %s', 'widget-for-eventbrite-api' ),
					'music_show_door_time'             => 'false',
					/* translators: %1$s time (e.g. Doors open: 18:30) %2$s time (e.g.Show starts: 19:30) */
					'music_show_door_time_label'       => esc_html__( 'Doors: %1$s | Show: %2$s', 'widget-for-eventbrite-api' ),
					'music_show_presented_by'          => 'false',
					'music_show_presented_by_label'    => '',
					'past_event_button'                => '',
					'popup'                            => 'false',
					'postponed_button'                 => '',
					'sales_ended_button'               => '',
					'series_id'                        => '',
					'show_availability'                => 0,
					'show_prices'                      => 'false',
					'sold_out_button'                  => '',
					'started_event_button'             => '',
					'tickets_at_the_door_button'       => '',
					'unavailable_button'               => '',
				);
				$defaults      = array_merge( $defaults, $defaults_gold );
			}
			if ( $wfea_fs->is_plan_or_trial( 'platinum' ) ) {
				$defaults_platinum = array(
					'search_box'        => 'false',
					'search_box_text'   => esc_html__( 'Search events', 'widget-for-eventbrite-api' ),
					'search_box_button' => esc_html__( 'Search', 'widget-for-eventbrite-api' ),
					'events_per_page'   => 0,
					'paginate_position' => 'both', // top, bottom, both, none
					'paged'             => 1,
					'get_artists_data'  => 'false',
				);
				$defaults          = array_merge( $defaults, $defaults_platinum );
			}
		}

		// Allow plugins/themes developer to filter the default arguments.
		return apply_filters( 'eawp_shortcode_default_args', $defaults );
	}

	public static function get_cal_locale() {
		$locale = str_replace( '_', '-', strtolower( get_locale() ) );
		$parts  = explode( '-', $locale );
		if ( $parts[0] == $parts[1] ) {
			$locale = $parts[0];
		}

		return apply_filters( 'wfea_cal_locale', $locale );
	}

	public function add_search_query__premium_only( $query ) {
		global $wfea_instance_counter;
		// check correct instance
		if ( 'wfea_instance_' . $wfea_instance_counter != $_REQUEST['wfea_s_target'] ) {
			return $query;
		}
		// check it is a valid nonce  for this instance
		if ( ! wp_verify_nonce( $_REQUEST['_wfea_nonce'], 'wfea_s_' . $wfea_instance_counter ) ) {
			return $query;
		}
		$query['search'] = '';
		$query_in = sanitize_text_field( $_REQUEST['wfea_s'] );
		if ( ! empty( $query_in ) ) {
			$delim = '/';
			$case  = 'i';
			if ( substr( $query_in, 0, 1 ) === '/' ) {
				//dont delimit
				$delim = '';
				$case  = '';
			}
			if ( false !== preg_match( $delim . $query_in . $delim, '' ) ) {
				$query['search'] = $delim . $query_in . $delim . $case;
			}
		}

		return $query;
	}

	public function add_shortcode() {
		add_filter('aioseo_conflicting_shortcodes', function($conflicting_shortcodes) {
			$conflicting_shortcodes['Display Eventbrite Events']='wfea';
			return $conflicting_shortcodes;
		});
		add_shortcode( 'wfea', array( $this, 'build_shortcode' ) );
	}

	public function build_shortcode( $initial_atts ) {
		$sc_atts = shortcode_atts( self::default_args(), $initial_atts, 'wfea' );

		if ( $this->freemius->can_use_premium_code__premium_only() ) {
			// any template that is not widget is premium
			/* alter divi templates as deprecated */
			if ( 'divi' == $sc_atts['layout'] ) {
				$sc_atts['layout'] = '1';
			}
			if ( 'divigrid' == $sc_atts['layout'] ) {
				$sc_atts['layout'] = 'grid';
			}
			if ( 'musical' == $sc_atts['layout'] ) {
				$sc_atts['layout'] = 'venue';
			}
		}
		$atts                    = array();
		$atts['booknow']         = $this->shortcode_bool( $sc_atts['booknow'] );
		$atts['booknow_text']    = wp_kses_post( urldecode( $sc_atts['booknow_text'] ) );
		$atts['cssid']           = sanitize_html_class( $sc_atts['cssid'] );
		$atts['css_class']       = sanitize_html_class( $sc_atts['css_class'] );
		$atts['date']            = $this->shortcode_bool( $sc_atts['date'] );
		$atts['debug']           = $this->shortcode_bool( $sc_atts['debug'] );
		$atts['display_private'] = false;
		$atts['excerpt']         = $this->shortcode_bool( $sc_atts['excerpt'] );
		$atts['layout']          = sanitize_text_field( $sc_atts['layout'] );
		$atts['length']          = (int) $sc_atts['length'];
		$atts['limit']           = (int) $sc_atts['limit'];
		$atts['newtab']          = $this->shortcode_bool( $sc_atts['newtab'] );
		$atts['order_by']        = sanitize_text_field( $sc_atts['order_by'] );
		$atts['readmore']        = $this->shortcode_bool( $sc_atts['readmore'] );
		$atts['readmore_text']   = wp_kses_post( urldecode( $sc_atts['readmore_text'] ) );
		$atts['status']          = sanitize_text_field( $sc_atts['status'] );
		$atts['thumb']           = $this->shortcode_bool( $sc_atts['thumb'] );
		$atts['thumb_align']     = sanitize_text_field( $sc_atts['thumb_align'] );
		$atts['thumb_default']   = esc_url( $sc_atts['thumb_default'] );
		$atts['thumb_original']  = $this->shortcode_bool( $sc_atts['thumb_original'] );
		$atts['thumb_width']     = (int) $sc_atts['thumb_width'];
		$atts['tickets']         = $this->shortcode_bool( $sc_atts['tickets'] );
		$atts['widgetwrap']      = $this->shortcode_bool( $sc_atts['widgetwrap'] );


		if ( $this->freemius->can_use_premium_code__premium_only() ) {

			$atts['category_id']                = sanitize_text_field( $sc_atts['category_id'] );
			$atts['content']                    = $this->shortcode_bool( $sc_atts['content'] );
			$atts['deduplicate']                = $this->shortcode_bool( $sc_atts['deduplicate'] );
			$atts['display_private']            = $this->shortcode_bool( $sc_atts['display_private'] );
			$atts['eb_id']                      = sanitize_text_field( $sc_atts['eb_id'] );
			$atts['filter_location']            = urldecode( sanitize_text_field( $sc_atts['filter_location'] ) );
			$atts['filter_title']               = urldecode( sanitize_text_field( $sc_atts['filter_title'] ) );
			$atts['first_of_series']            = $this->shortcode_bool( $sc_atts['first_of_series'] );
			$atts['format_id']                  = sanitize_text_field( $sc_atts['format_id'] );
			$atts['location']                   = sanitize_text_field( $sc_atts['location'] );
			$atts['location_title']             = wp_kses_post( urldecode( $sc_atts['location_title'] ) );
			$atts['long_description']           = $this->shortcode_bool( $sc_atts['long_description'] );
			$atts['no_events_found_text']       = wp_kses_post( urldecode( $sc_atts['no_events_found_text'] ) );
			$atts['online_events_address_text'] = sanitize_text_field( $sc_atts['online_events_address_text'] );
			$atts['online_events_only']         = $this->shortcode_bool( $sc_atts['online_events_only'] );
			$atts['organization_id']            = (int) $sc_atts['organization_id'];
			$atts['organizer_id']               = (int) $sc_atts['organizer_id'];
			$atts['search']                     = urldecode( sanitize_text_field( $sc_atts['search'] ) );
			$atts['show_date_time']             = $this->shortcode_bool( $sc_atts['show_date_time'] );
			$atts['show_end_time']              = $this->shortcode_bool( $sc_atts['show_end_time'] );
			$atts['start_date_range_start']     = sanitize_text_field( $sc_atts['start_date_range_start'] );
			$atts['start_date_range_end']       = sanitize_text_field( $sc_atts['start_date_range_end'] );
			$atts['style']                      = sanitize_text_field( $sc_atts['style'] );
			$atts['subcategory_id']             = sanitize_text_field( $sc_atts['subcategory_id'] );
			$atts['venue_id']                   = sanitize_text_field( $sc_atts['venue_id'] );
			$atts['venue_name']                 = $this->shortcode_bool( $sc_atts['venue_name'] );


			if ( $this->freemius->is_plan_or_trial( 'gold' ) ) {
				$atts['api_key']                          = sanitize_text_field( $sc_atts['api_key'] );
				$atts['cal_default_view']                 = sanitize_text_field( $sc_atts['cal_default_view'] );
				$atts['cal_list_days']                    = (int) $sc_atts['cal_list_days'];
				$atts['cal_list_header_center']           = sanitize_text_field( $sc_atts['cal_list_header_center'] );
				$atts['cal_list_header_left']             = sanitize_text_field( $sc_atts['cal_list_header_left'] );
				$atts['cal_list_header_right']            = sanitize_text_field( $sc_atts['cal_list_header_right'] );
				$atts['canceled_event_button']            = wp_kses_post( urldecode( $sc_atts['canceled_event_button'] ) );
				$atts['coming_soon_button']               = wp_kses_post( urldecode( $sc_atts['coming_soon_button'] ) );
				$atts['direct_to_external']               = $this->shortcode_bool( $sc_atts['direct_to_external'] );
				$atts['link_custom_page']                 = sanitize_text_field( $sc_atts['link_custom_page'] );
				$atts['music_show_age_restriction']       = $this->shortcode_bool( $sc_atts['music_show_age_restriction'] );
				$atts['music_show_age_restriction_label'] = wp_kses_post( urldecode( $sc_atts['music_show_age_restriction_label'] ) );
				$atts['music_show_door_no_time_label']    = wp_kses_post( urldecode( $sc_atts['music_show_door_no_time_label'] ) );
				$atts['music_show_door_time']             = $this->shortcode_bool( $sc_atts['music_show_door_time'] );
				$atts['music_show_door_time_label']       = wp_kses_post( urldecode( $sc_atts['music_show_door_time_label'] ) );
				$atts['music_show_presented_by']          = $this->shortcode_bool( $sc_atts['music_show_presented_by'] );
				$atts['music_show_presented_by_label']    = wp_kses_post( urldecode( $sc_atts['music_show_presented_by_label'] ) );
				$atts['past_event_button']                = wp_kses_post( urldecode( $sc_atts['past_event_button'] ) );
				$atts['popup']                            = $this->shortcode_bool( $sc_atts['popup'] );
				$atts['postponed_button']                 = wp_kses_post( urldecode( $sc_atts['postponed_button'] ) );
				$atts['sales_ended_button']               = wp_kses_post( urldecode( $sc_atts['sales_ended_button'] ) );
				$atts['series_id']                        = sanitize_text_field( $sc_atts['series_id'] );
				$atts['show_availability']                = (int) $sc_atts['show_availability'];
				$atts['show_prices']                      = $this->shortcode_bool( $sc_atts['show_prices'] );
				$atts['sold_out_button']                  = wp_kses_post( urldecode( $sc_atts['sold_out_button'] ) );
				$atts['started_event_button']             = wp_kses_post( urldecode( $sc_atts['started_event_button'] ) );
				$atts['tickets_at_the_door_button']       = wp_kses_post( urldecode( $sc_atts['tickets_at_the_door_button'] ) );
				$atts['unavailable_button']               = wp_kses_post( urldecode( $sc_atts['unavailable_button'] ) );
			}
			if ( $this->freemius->is_plan_or_trial( 'platinum' ) ) {
				$atts['search_box']        = $this->shortcode_bool( $sc_atts['search_box'] );
				$atts['search_box_text']   = sanitize_text_field( $sc_atts['search_box_text'] );
				$atts['search_box_button'] = sanitize_text_field( $sc_atts['search_box_button'] );
				$atts['events_per_page']   = (int) $sc_atts['events_per_page'];
				$atts['paginate_position'] = sanitize_text_field( $sc_atts['paginate_position'] );
				$atts['paged']             = (int) $sc_atts['paged'];
				$atts['get_artists_data']  = $this->shortcode_bool( $sc_atts['get_artists_data'] );;
			}

		}


		// Query arguments.
		$query = array(
			'nopaging' => true,
			'limit'    => $atts['limit'],
			'layout'   => $atts['layout'],

		);
		if ( ! empty( $atts['status'] ) ) {
			$atts['status']  = strtolower( $atts['status'] );
			$atts['status']  = str_replace( 'cancelled', 'canceled', $atts['status'] );
			$query['status'] = $atts['status'];
		}
		if ( ! empty( $atts['order_by'] ) ) {
			if ( 'asc' === strtolower( $atts['order_by'] ) ) {
				$query['order_by'] = 'start_asc';
			} elseif ( 'desc' === strtolower( $atts['order_by'] ) ) {
				$query['order_by'] = 'start_desc';
			} elseif ( 'created_desc' === strtolower( $atts['order_by'] ) ) {
				$query['order_by'] = 'created_desc';
			} elseif ( 'created_asc' === strtolower( $atts['order_by'] ) ) {
				$query['order_by'] = 'created_asc';
			} elseif ( 'published_desc' === strtolower( $atts['order_by'] ) ) {
				$query['order_by'] = 'published_desc';
			} elseif ( 'published_asc' === strtolower( $atts['order_by'] ) ) {
				$query['order_by'] = 'published_asc';
			}

		}
		$query['thumb_original']  = $atts['thumb_original'];
		$query['display_private'] = $atts['display_private'];
		if ( $this->freemius->can_use_premium_code__premium_only() ) {
			$query['long_description'] = $atts['long_description'];
			if ( ! empty( $atts['api_key'] ) ) {
				$query['token'] = $atts['api_key'];
			}
			if ( ! empty( $atts['category_id'] ) ) {
				$query['category_id'] = $atts['category_id'];
			}
			if ( ! empty( $atts['deduplicate'] ) ) {
				$query['deduplicate'] = $atts['deduplicate'];
			}
			if ( ! empty( $atts['eb_id'] ) ) {
				$query['ID'] = $atts['eb_id'];
			} else if ( isset( $_GET['wfea_eb_id'] ) && ! empty ( $_GET['wfea_eb_id'] ) ) {

				$query['ID'] = $_GET['wfea_eb_id'];
			} else {
				$qid = get_query_var( 'wfea_eb_id' );
				if ( ! empty( $qid ) ) {
					$query['ID'] = $qid;
				}
			}
			if ( ! empty( $atts['filter_title'] ) ) {
				$delim = '/';
				if ( substr( $atts['filter_title'], 0, 1 ) === '/' ) {
					//dont delimit
					$delim = '';
				}
				if ( false === preg_match( $delim . $atts['filter_title'] . $delim, '' ) ) {
					esc_html_e( '*** Invalid regex used in your shortcode filter_title option *** ', 'widget-for-eventbrite-api' );
				} else {
					$query['filter_title'] = $delim . $atts['filter_title'] . $delim;
				}
			}
			if ( ! empty( $atts['filter_location'] ) ) {
				$delim = '/';
				if ( substr( $atts['filter_location'], 0, 1 ) === '/' ) {
					//dont delimit
					$delim = '';
				}
				if ( false === preg_match( $delim . $atts['filter_location'] . $delim, '' ) ) {
					esc_html_e( '*** Invalid regex used in your shortcode filter_location option *** ', 'widget-for-eventbrite-api' );
				} else {
					$query['filter_location'] = $delim . $atts['filter_location'] . $delim;
				}
			}
			if ( ! empty( $atts['first_of_series'] ) ) {
				$query['first_of_series'] = $atts['first_of_series'];
			}
			if ( ! empty( $atts['format_id'] ) ) {
				$query['format_id'] = $atts['format_id'];
			}
			if ( ! empty( $atts['no_events_found_text'] ) ) {
				$query['no_events_found_text'] = $atts['no_events_found_text'];
			}
			if ( ! empty( $atts['online_events_address_text'] ) ) {
				$query['online_events_address_text'] = $atts['online_events_address_text'];
			}
			if ( ! empty( $atts['online_events_only'] ) ) {
				$query['online_events_only'] = $atts['online_events_only'];
			}
			if ( ! empty( $atts['organization_id'] ) ) {
				$query['organization_id'] = $atts['organization_id'];
			}
			if ( ! empty( $atts['organizer_id'] ) ) {
				$query['organizer_id'] = $atts['organizer_id'];
			}
			if ( ! empty( $atts['search'] ) ) {
				$delim = '/';
				$case  = 'i';
				if ( substr( $atts['search'], 0, 1 ) === '/' ) {
					//dont delimit
					$delim = '';
					$case  = '';
				}
				if ( false === preg_match( $delim . $atts['search'] . $delim, '' ) ) {
					esc_html_e( '*** Invalid regex used in your shortcode search option *** ', 'widget-for-eventbrite-api' );
				} else {
					$query['search'] = $delim . $atts['search'] . $delim . $case;
				}
			}
			if ( ! empty( $atts['show_end_time'] ) ) {
				$query['show_end_time'] = $atts['show_end_time'];
			}
			if ( ! empty( $atts['start_date_range_start'] ) && empty( $atts['start_date_range_end'] ) ) {
				$atts['start_date_range_end'] = '2999';
			}
			if ( ! empty( $atts['start_date_range_start'] ) && ! empty( $atts['start_date_range_end'] ) ) {
				$query['start_date.range_start'] = date( 'Y-m-d', strtotime( $atts['start_date_range_start'] ) );
				$query['start_date.range_end']   = date( 'Y-m-d', strtotime( $atts['start_date_range_end'] ) );
			}
			if ( ! empty( $atts['subcategory_id'] ) ) {
				$query['subcategory_id'] = $atts['subcategory_id'];
			}
			if ( ! empty( $atts['venue_id'] ) ) {
				$query['venue_id'] = $atts['venue_id'];
			}
			if ( ! empty( $atts['venue_name'] ) ) {
				$query['venue_name'] = $atts['venue_name'];
			}

			if ( $this->freemius->is_plan_or_trial( 'gold' ) ) {
				if ( ! empty( $atts['canceled_event_button'] ) ) {
					$query['canceled_event_button'] = $atts['canceled_event_button'];
				}
				if ( ! empty( $atts['coming_soon_button'] ) ) {
					$query['coming_soon_button'] = $atts['coming_soon_button'];
				}
				if ( ! empty( $atts['direct_to_external'] ) ) {
					$query['direct_to_external'] = $atts['direct_to_external'];
				}
				if ( ! empty( $atts['link_custom_page'] ) ) {
					$query['link_custom_page'] = $atts['link_custom_page'];
					$atts['popup']             = false;   //  stop popup processing if liking to internal page
				}
				if ( ! empty( $atts['past_event_button'] ) ) {
					$query['past_event_button'] = $atts['past_event_button'];
				}
				if ( ! empty( $atts['postponed_button'] ) ) {
					$query['postponed_button'] = $atts['postponed_button'];
				}
				if ( ! empty( $atts['sales_ended_button'] ) ) {
					$query['sales_ended_button'] = $atts['sales_ended_button'];
				}
				if ( ! empty( $atts['series_id'] ) ) {
					$query['series_id'] = $atts['series_id'];
				}
				if ( ! empty( $atts['show_availability'] ) ) {
					$query['show_availability'] = $atts['show_availability'];
				}
				if ( ! empty( $atts['show_prices'] ) ) {
					$query['show_prices'] = $atts['show_prices'];
				}
				if ( ! empty( $atts['sold_out_button'] ) ) {
					$query['sold_out_button'] = $atts['sold_out_button'];
				}
				if ( ! empty( $atts['started_event_button'] ) ) {
					$query['started_event_button'] = $atts['started_event_button'];
				}
				if ( ! empty( $atts['tickets_at_the_door_button'] ) ) {
					$query['tickets_at_the_door_button'] = $atts['tickets_at_the_door_button'];
				}
				if ( ! empty( $atts['unavailable_button'] ) ) {
					$query['unavailable_button'] = $atts['unavailable_button'];
				}
			}
			if ( $this->freemius->is_plan_or_trial( 'platinum' ) ) {
				if ( ! empty( $atts['search_box'] ) ) {
					$query['search_box'] = $atts['search_box'];
				}
				if ( ! empty( $atts['search_box_text'] ) ) {
					$query['search_box_text'] = $atts['search_box_text'];
				}
				if ( ! empty( $atts['search_box_button'] ) ) {
					$query['search_box_button'] = $atts['search_box_button'];
				}
				if ( ! empty( $atts['events_per_page'] ) ) {
					$query['events_per_page'] = $atts['events_per_page'];
					$query['nopaging']        = false;
				}
				if ( ! empty( $atts['paginate_position'] ) ) {
					$query['paginate_position'] = $atts['paginate_position'];
				}
				if ( ! empty( $atts['paged'] ) ) {
					$query['paged'] = $atts['paged'];
				}
				if ( ! empty( $atts['get_artists_data'] ) ) {
					$query['get_artists_data'] = $atts['get_artists_data'];
				}
			}
		}


		global $wfea_instance_counter;
		$wfea_instance_counter ++;

		// Allow plugins/themes developer to filter the query.
		$query = apply_filters( 'eawp_shortcode_query_arguments', $query );
		$atts  = apply_filters( 'eawp_shortcode_atts', $atts );


		// Perform the query.

		$events = new Eventbrite_Query( $query );

		$html      = '';
		$admin_msg = '<div class="wfea error">' . esc_html__( '[Display Eventbrite Plugin] Admin Notice! ( this shows to admins only ): ', 'widget-for-eventbrite-api' ) . '</div>';

		if ( $this->freemius->can_use_premium_code__premium_only() ) {
			if ( current_user_can( 'manage_options' ) ) {
				$failed_batches = as_get_scheduled_actions(
					array(
						'group'  => 'wfea',
						'status' => ActionScheduler_Store::STATUS_FAILED,
					)
				);
				if ( ! empty( $failed_batches ) ) {
					$html .= $admin_msg .
					         '<div class="wfea error">' .
					         esc_html( 'Background processing has some failed events: ', 'widget_for_eventbrite_api' ) .
					         '<a href="' . admin_url( 'tools.php?page=action-scheduler&status=failed' ) . '">' .
					         esc_html( 'Review the logs for details', 'widget_for_eventbrite_api' ) .
					         '</a>' .
					         '</div>';
				}
			}
		}
		if ( is_wp_error( $events->api_results ) ) {
			if ( current_user_can( 'manage_options' ) ) {

				$error_string = $this->utilities->get_api_error_string( $events->api_results );
				$html         .= $admin_msg .
				                 '<div class="wfea error">' . $error_string . '</div>';
				if ( $atts['debug'] ) {
					$html .= $this->get_debug_output( $events );
				}
			}
		} else {

			ob_start();
			$this->check_valid_att( $initial_atts );

			$theme           = wp_get_theme();
			$template_loader = new Template_Loader();
			$template_loader->set_template_data(
				array(
					'template_loader' => $template_loader,
					'events'          => $events,
					'args'            => $atts,
					'template'        => strtolower( $theme->template ),
					'plugin_name'     => $this->plugin_name,
					'utilities'       => $this->utilities,
					'unique_id'       => uniqid(),
					'instance'        => $wfea_instance_counter,
					'event'           => new stdClass(),
				)
			);

			$template_found = $template_loader->get_template_part( 'layout_' . $atts['layout'] );


			if ( false == $template_found ) {
				if ( current_user_can( 'manage_options' ) ) {

					$layouts    = 'widget';
					$plan_title = esc_html__( 'Free', 'widget-for-eventbrite-api' );
					if ( $this->freemius->can_use_premium_code__premium_only() ) {
						$plan = $this->freemius->get_plan();
						if ( false !== $plan ) {
							$plan_title = $plan->title;
							switch ( $plan->name ) {
								case 'silver':
									$layouts = 'widget,1,grid';
									break;
								case 'premium':  // 2017
								case 'prolegacy2019':
								case 'gold':
									$layouts = 'widget,1,grid,venue,cal,cal_list,slider';
									break;
								case 'platinum':
									$layouts = 'widget,1,grid,venue,cal,cal_list,slider';
									break;
							}
						}
					}
					$err_msg = $admin_msg .
					           '<div class="wfea error">' .
					           esc_html__( 'Selected LAYOUT="', 'widget_for_eventbrite_api' ) .
					           esc_html( $atts['layout'] ) .
					           esc_html__( '" Not found in any paths. Your plan is ', 'widget_for_eventbrite_api' ) .
					           esc_html( $plan_title ) .
					           esc_html__( ' and includes these layouts ', 'widget_for_eventbrite_api' ) .
					           esc_html( $layouts ) .
					           esc_html__( ' and any custom developed layouts you have made.', 'widget_for_eventbrite_api' ) . '<br><br>' .
					           esc_html__( 'Paths checked are:', 'widget_for_eventbrite_api' ) . '<br>' .
					           implode( '<br>', $template_loader->get_file_paths() );
					'</div>';
					echo $err_msg;
				}
			}


			if ( $atts['debug'] ) {
				echo wp_kses_post( $this->get_debug_output( $events ) );
			}
			$html .= ob_get_clean();
			$html = apply_filters( 'eawp_shortcode_markup', $html );

			// Restore original Post Data.
			wp_reset_postdata();
		}

		return $html;
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 */
	public function enqueue_scripts() {

		/**  @var \Freemius $wfea_fs freemius SDK. */
		global $wfea_fs;

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/frontend.js', array( 'jquery' ), $this->version, false );


		if ( $wfea_fs->is__premium_only() ) {
			if ( $wfea_fs->can_use_premium_code() ) {
				wp_register_script( $this->plugin_name . '-frontend-pro', plugin_dir_url( __FILE__ ) . 'js/frontend__premium_only.js', array(
					'jquery',
					'jquery-ui-dialog'
				), $this->version, false );
				// ebscipt for popup
				wp_register_script( $this->plugin_name . '-eb-script', 'https://www.eventbrite.com/static/widgets/eb_widgets.js' );
				wp_register_script( $this->plugin_name . '-eb-popup', plugin_dir_url( __FILE__ ) . 'js/eb_popup.js', array( 'jquery' ), $this->version, false );
				// Calendar scripts.
				wp_register_script( $this->plugin_name . '-qtip', dirname( plugin_dir_url( __FILE__ ) ) . '/includes/vendor/qtip2/qtip2/dist/jquery.qtip.min.js', array( 'jquery' ), '3.0.3', false );

				wp_register_script( $this->plugin_name . '-moment', dirname( plugin_dir_url( __FILE__ ) ) . '/includes/vendor/moment/moment/min/moment.min.js', array( 'jquery' ), '1.0', false );
				wp_register_script( $this->plugin_name . '-fullcalendar', dirname( plugin_dir_url( __FILE__ ) ) . '/includes/vendor/fullcalendar/fullcalendar/dist/fullcalendar.min.js', array(
					'jquery',
					$this->plugin_name . '-moment'
				), '3.9.0', false );
				wp_register_script( $this->plugin_name . '-locale', dirname( plugin_dir_url( __FILE__ ) ) . '/includes/vendor/fullcalendar/fullcalendar/dist/locale-all.js', array(
					'jquery',
					$this->plugin_name . '-fullcalendar'
				), '3.9.0', false );
				wp_register_script( $this->plugin_name . '-calendar', plugin_dir_url( __FILE__ ) . 'js/calendar.js', array(
					'jquery',
					$this->plugin_name . '-moment',
					$this->plugin_name . '-fullcalendar',
					$this->plugin_name . '-qtip',
					$this->plugin_name . '-eb-popup'
				), $this->version, false );
				wp_register_script( $this->plugin_name . '-featherlight-js', dirname( plugin_dir_url( __FILE__ ) ) . '/includes/vendor/noelboss/featherlight/release/featherlight.min.js', array(
					'jquery',
				), '1.7.14', false );
				wp_register_script( $this->plugin_name . '-slider', plugin_dir_url( __FILE__ ) . 'js/slider.js', array( 'jquery' ), $this->version, false );
			}
		}
	}

	/**
	 * Register the stylesheets for the frontend.
	 */
	public function enqueue_styles() {
		/**  @var \Freemius $wfea_fs freemius SDK. */
		global $wfea_fs;

		$options = get_option( 'widget-for-eventbrite-api-settings' );
		if ( ! isset( $options['plugin-css'] ) || $options['plugin-css'] ) { // need to check not set as older version didn't have this option
			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/frontend.css', array(), $this->version, 'all' );
		}

		if ( $wfea_fs->is__premium_only() ) {
			if ( $wfea_fs->can_use_premium_code() ) {
				// Calendar styles.
				wp_register_style( $this->plugin_name . '-fullcalendar-css', dirname( plugin_dir_url( __FILE__ ) ) . '/includes/vendor/fullcalendar/fullcalendar/dist/fullcalendar.min.css', array(), '3.9.0', 'all' );
				wp_register_style( $this->plugin_name . '-fullcalendar-print-css', dirname( plugin_dir_url( __FILE__ ) ) . '/includes/vendor/fullcalendar/fullcalendar/dist/fullcalendar.print.min.css', array(), '3.9.0', 'print' );
				wp_register_style( $this->plugin_name . '-qtip-css', dirname( plugin_dir_url( __FILE__ ) ) . '/includes/vendor/qtip2/qtip2/dist/jquery.qtip.min.css', array(), '3.0.3', 'all' );
				wp_register_style( $this->plugin_name . '-featherlight-css', dirname( plugin_dir_url( __FILE__ ) ) . '/includes/vendor/noelboss/featherlight/release/featherlight.min.css', array(), '1.7.14', 'all' );
			}
		}
	}

	public function jetpack_photon_skip_for_url( $skip, $image_url ) {

		$banned_host_patterns = array( '/^img\.evbuc\.com$/' );
		$host                 = wp_parse_url( $image_url, PHP_URL_HOST );

		foreach ( $banned_host_patterns as $banned_host_pattern ) {
			if ( 1 === preg_match( $banned_host_pattern, $host ) ) {
				return true;
			}
		}

		return $skip;

	}

	public function process_search__premium_only() {
		// check is a search.
		if ( ! isset( $_REQUEST['wfea_s'] ) || ! isset( $_REQUEST['_wfea_nonce'] ) || ! isset( $_REQUEST['wfea_s_target'] ) ) {
			return;
		}
		// check if the referer is self ?

		// filter atts

		add_filter( 'eawp_default_query_arguments', array( $this, 'add_search_query__premium_only' ) );
		add_filter( 'eawp_shortcode_query_arguments', array( $this, 'add_search_query__premium_only' ) );


	}

	public function register_image_size() {
		add_image_size( 'eaw-thumbnail', 45, 45, true );
	}

	public function wfea_download_ics__premium_only() {
		if ( ! wp_doing_ajax() ) {
			return;
		}

		if ( ! isset( $_POST['wfea_ical_nonce'] )
		     || ! wp_verify_nonce( $_POST['wfea_ical_nonce'], 'wfea_ical_request' )
		) {
			return;
		}

		header( 'Content-Type: text/calendar; charset=utf-8' );
		header( 'Content-Disposition: attachment; filename=calendar.ics' );

		$ics = new ICS( array(
			'location'    => sanitize_text_field( $_POST['location'] ),
			'description' => sanitize_text_field( $_POST['description'] ),
			'dtstart'     => sanitize_text_field( $_POST['dtstart'] ),
			'dtend'       => sanitize_text_field( $_POST['dtend'] ),
			'summary'     => sanitize_text_field( $_POST['summary'] ),
			'url'         => esc_url( $_POST['url'] ),
		) );
		echo $ics->to_string();
		die();
	}

	private function shortcode_bool( $att ) {
		if ( 'true' === $att ) {
			$att = true;
		} else {
			$att = false;
		}

		return (bool) $att;
	}

	private function get_debug_output( $events ) {
		return '<h2>' . esc_html__( '--- DEBUG OUTPUT ---', 'widget-for-eventbrite-api' ) . '</h2><pre>' . print_r( $events->api_results, true ) . '</pre>';
	}

	private function check_valid_att( $atts ) {
		if ( ! is_array( $atts ) ) {
			return;
		}
		$defaults = self::default_args();
		foreach ( $atts as $att => $value ) {
			if ( ! isset( $defaults[ $att ] ) ) {
				$message = esc_html__( '[Display Eventbrite Plugin] Selected attribute: [', 'widget-for-eventbrite-api' ) .
				           esc_attr( $att ) .
				           esc_html__( '] is not valid - maybe a typo or maybe not included in your plan, refer to documentation', 'widget-for-eventbrite-api' );
				if ( isset( $atts['debug'] ) && $atts['debug'] ) {
					echo '<div class="error">' . $message . '</div>';
					trigger_error( $message, E_USER_NOTICE );
				}
			}
		}
	}
}
