<?php

namespace WidgetForEventbriteAPI\Includes;

use WP_Query;

class Eventbrite_Query extends WP_Query {
	/**
	 * Results from the API call. Includes up to 50 objects plus pagination info.
	 *
	 * @var object
	 */
	public $api_results;

	/**
	 * Constructor.
	 *
	 * Sets up the Eventbrite query.
	 *
	 * @access public
	 *
	 * @param string $query URL query string.
	 */
	public function __construct( $query = '' ) {
		// Process any query args from the URL.
		$query = $this->process_query_args( $query );
		// Assign hooks.
		remove_filter( 'the_content', 'wpautop' );

		// Put our query in motion.
		$this->query( $query );

	}

	/**
	 * Handle any query args that come from the requested URL.
	 *
	 * @access protected
	 *
	 * @param mixed $query Query string.
	 *
	 * @return array Query arguments
	 */
	protected function process_query_args( $query ) {
		// Handle requests for paged events.


		/**
		 * @var \Freemius $wfea_fs Object for freemius.
		 */
		global $wfea_fs;
		if ( $wfea_fs->is__premium_only() ) {
			global $wfea_instance_counter;
			if ( isset( $_GET[ 'wfea_page_no_' . $wfea_instance_counter ] ) ) {
				$query['paged'] = (int) $_GET[ 'wfea_page_no_' . $wfea_instance_counter ];
			}

			// not sure if the below is any use

			// Filter by organizer ID if an "author archive" (organizer events) was requested.
			$organizer_id = get_query_var( 'organizer_id' );
			if ( empty( $query['organizer_id'] ) && ! empty( $organizer_id ) ) {
				$query['organizer_id'] = (int) $organizer_id;
			}

			// Filter by venue ID if a venue archive (all events at a certain venue) was requested.
			$venue_id = get_query_var( 'venue_id' );
			if ( empty( $query['venue_id'] ) && ! empty( $venue_id ) ) {
				$query['venue_id'] = (int) $venue_id;
			}

			// Filter by category ID if a category archive (all events in a certain category) was requested.
			$category_id = get_query_var( 'category_id' );
			if ( empty( $query['category_id'] ) && ! empty( $category_id ) ) {
				$query['category_id'] = (int) $category_id;
			}

			// Filter by subcategory ID if a subcategory archive (all events in a certain subcategory) was requested.
			$subcategory_id = get_query_var( 'subcategory_id' );
			if ( empty( $query['subcategory_id'] ) && ! empty( $subcategory_id ) ) {
				$query['subcategory_id'] = (int) $subcategory_id;
			}

			// Filter by format ID if a format archive (all events in a certain format) was requested.
			$format_id = get_query_var( 'format_id' );
			if ( empty( $query['format_id'] ) && ! empty( $format_id ) ) {
				$query['format_id'] = (int) $format_id;
			}
		}

		return $query;
	}

	/**
	 * Retrieve the posts based on query variables.
	 *
	 * @access public
	 *
	 * @return array List of posts.
	 */
	public function get_posts() {
		// Set up query variables.
		$this->parse_query();

		// Restore `paged` if changed to `page` (in the case of index pagination).
		if ( ! empty( $this->query_vars['page'] ) ) {
			$this->query_vars['paged'] = $this->query_vars['page'];
			unset( $this->query_vars['page'] );
		}
		// remove paged if a search query


		// Set any required parameters for the API request based on the query vars.
		$params = $this->set_api_params();


		for ( $i = 1; $i <= 10; $i ++ ) {
			$this->api_results = Eventbrite_Manager::$instance->get_organizations_events( $params );
			if ( ! is_wp_error( $this->api_results ) ) {
				break;
			}
			usleep( 500000 );
			if ( defined( 'WP_DEBUG' ) && true === WP_DEBUG && is_wp_error( $this->api_results ) ) {
				error_log( '[' . date( "F j, Y, g:i a e O" ) . '] ' .
				           print_r( $this->api_results->errors, true ) . ' attempt: ' . $i
				);
			}
		}

		if ( is_wp_error( $this->api_results ) ) {
			return array();
		}


		// Do any post-API query processing.
		$this->post_api_filters();

		// Set properties based on the results.
		$this->set_properties();

		// Return what we have for posts.
		// examine if we have any paid events
		// @TODO paid events only processing
		/*
		foreach ( $this->api_results->events as $event) {
			if ( false == $event->is_free ) {
				$version_history = get_option( 'widget-for-eventbrite-api-version' );
				if ( $version_history ) {
					$version_history['paid'] = true;
					update_option( 'widget-for-eventbrite-api-version', $version_history );
					break;
				}
			}
		}
		*/

		return $this->posts;
	}

	/**
	 * Determine parameters for an API call.
	 *
	 * @access protected
	 *
	 * @return array API call parameters
	 */
	protected function set_api_params() {
		$params = array();

		// Add 'page' parameter if we need events above the first 50.


		// Adjust status for private events if necessary.
		if ( ! empty( $this->query_vars['status'] ) ) {
			$params['status'] = $this->query_vars['status'];
		}
		if ( ! empty( $this->query_vars['start_date.range_start'] ) ) {
			$params['start_date.range_start'] = $this->query_vars['start_date.range_start'];
		}
		if ( ! empty( $this->query_vars['start_date.range_end'] ) ) {
			$params['start_date.range_end'] = $this->query_vars['start_date.range_end'];
		}

		if ( isset ( $this->query_vars['token'] ) && ! empty( $this->query_vars['token'] ) ) {
			$params['token'] = $this->query_vars['token'];
		}

		if ( isset ( $this->query_vars['organization_id'] ) && ! empty( $this->query_vars['organization_id'] ) ) {
			$params['organization_id'] = $this->query_vars['organization_id'];
		}

		return $params;
	}

	/**
	 * Process any remaining internal query parameters. These are parameters that are specific to Eventbrite_Query, not the API calls.
	 *
	 * @access protected
	 */
	protected function post_api_filters() {
		/**
		 * @var \Freemius $wfea_fs Object for freemius.
		 */
		global $wfea_fs;
		if ( $wfea_fs->can_use_premium_code__premium_only() ) {
			// use long description rather than summary
			if ( isset( $this->query_vars['long_description'] ) && true === $this->query_vars['long_description'] ) {
				$this->get_descriptions__premium_only();
			}
			if ( isset( $this->query_vars['get_artists_data'] ) && true === $this->query_vars['get_artists_data'] ) {
				$this->get_performances__premium_only();
			}
		}

		$this->api_results->events = apply_filters(
			'wfea_api_results',
			$this->api_results->events,
			$this->query_vars
		);

		// Filter out private events: 'display_private'
		if ( $wfea_fs->is__premium_only() ) {
			if ( ! isset( $this->query_vars['display_private'] ) ) {
				$this->query_vars['display_private'] = false;
			}
		} else {
			$this->query_vars['display_private'] = false;
		}

		$this->api_results->events = array_filter( $this->api_results->events, array(
			$this,
			'filter_by_display_private'
		) );

		if ( $wfea_fs->is__premium_only() ) {
			//de duplicate
			if ( isset( $this->query_vars['deduplicate'] ) && true === $this->query_vars['deduplicate'] ) {
				$event_ids                 = array_map( function ( $e ) {
					return $e->ID;
				}, $this->api_results->events );
				$this->api_results->events = array_filter(
					$this->api_results->events,
					function ( $e, $k ) use ( $event_ids ) {
						$search = array_keys( $event_ids, $e->ID );
						$a      = ( $k == $search[0] );

						return $k == $search[0];
					},
					ARRAY_FILTER_USE_BOTH
				);
			}
		}

		// Do nothing if API results were empty, false, or an error.
		if ( empty( $this->api_results ) || is_wp_error( $this->api_results ) ) {
			return false;
		}
		if ( $wfea_fs->is__premium_only() ) {
			//  Replace url with external if direct
			if ( isset( $this->query_vars['direct_to_external'] ) && $this->query_vars['direct_to_external'] ) {

				$this->api_results->events = array_map(
					function ( $event ) {
						if ( ! isset( $event->is_externally_ticketed ) || ! $event->is_externally_ticketed ||
						     ! isset( $event->external_ticketing->external_url ) || empty( $event->external_ticketing->external_url )
						) {
							return $event;
						}
						$event->url = $event->external_ticketing->external_url;

						return $event;
					},
					$this->api_results->events
				);
			}
			// Filter title by reqex
			if ( isset( $this->query_vars['filter_title'] ) && ! empty( $this->query_vars['filter_title'] ) ) {
				$this->api_results->events = array_filter( $this->api_results->events, array(
					$this,
					'filter_by_title__premium_only'
				) );
			}

			// add locations to online events
			if ( isset( $this->query_vars['online_events_address_text'] ) && ! empty( $this->query_vars['online_events_address_text'] ) ) {
				$this->add_address_to_events__premium_only();
			}


			// Filter location by reqex
			if ( isset( $this->query_vars['filter_location'] ) && ! empty( $this->query_vars['filter_location'] ) ) {
				$this->api_results->events = array_filter( $this->api_results->events, array(
					$this,
					'filter_by_location__premium_only'
				) );
			}
		}

		// Filter out specified IDs: 'post__not_in'
		if ( isset( $this->query_vars['post__not_in'] ) && is_array( $this->query_vars['post__not_in'] ) ) {
			$this->api_results->events = array_filter( $this->api_results->events, array(
				$this,
				'filter_by_post_not_in'
			) );
		}
		if ( $wfea_fs->is__premium_only() ) {
			// Filter by organizer: 'organizer_id'
			if ( isset( $this->query_vars['organizer_id'] ) && is_integer( $this->query_vars['organizer_id'] ) ) {
				$this->api_results->events = array_filter( $this->api_results->events, array(
					$this,
					'filter_by_organizer__premium_only'
				) );
			}

			// Filter by venue: 'venue_id'
			if ( isset( $this->query_vars['venue_id'] ) ) {
				$this->api_results->events = array_filter( $this->api_results->events, array(
					$this,
					'filter_by_venue__premium_only'
				) );
			}

			// Filter by category: 'category_id'
			if ( isset( $this->query_vars['category_id'] ) ) {
				$this->api_results->events = array_filter( $this->api_results->events, array(
					$this,
					'filter_by_category__premium_only'
				) );
			}

			// Filter by subcategory: 'subcategory_id'
			if ( isset( $this->query_vars['subcategory_id'] ) ) {
				$this->api_results->events = array_filter( $this->api_results->events, array(
					$this,
					'filter_by_subcategory__premium_only'
				) );
			}

			// Filter by format: 'format_id'
			if ( isset( $this->query_vars['format_id'] ) ) {
				$this->api_results->events = array_filter( $this->api_results->events, array(
					$this,
					'filter_by_format__premium_only'
				) );
			}


			// Perform search  of title, location, description: 'search'
			if ( isset( $this->query_vars['search'] ) ) {
				$this->api_results->events = array_filter( $this->api_results->events, array(
					$this,
					'search__premium_only'
				) );
			}

			// Remove all but the first in a series
			if ( isset( $this->query_vars['first_of_series'] ) && $this->query_vars['first_of_series'] ) {
				$series = array();
				foreach ( $this->api_results->events as $event ) {
					if ( ! isset( $event->series_id ) || empty( $event->series_id ) ) {
						$event->series_first = true;
					} else {
						if ( ! in_array( $event->series_id, $series ) ) {
							$series[]            = $event->series_id;
							$event->series_first = true;
						} else {
							$event->series_first = false;
						}
					}
				}
				$this->api_results->events = array_filter( $this->api_results->events, function ( $array ) {
					return $array->series_first;
				} );
			}

			// Filter online_only: 'online_events_only'
			if ( isset( $this->query_vars['online_events_only'] ) && ( $this->query_vars['online_events_only'] ) ) {
				$this->api_results->events = array_filter( $this->api_results->events, array(
					$this,
					'filter_online_only__premium_only'
				) );
			}
		}
		//   use original logo url
		if ( isset( $this->query_vars['thumb_original'] ) && true === $this->query_vars['thumb_original'] ) {
			foreach ( $this->api_results->events as $key => $event ) {
				if ( isset( $event->logo->original->url ) && ! empty( isset( $event->logo->original->url ) ) ) {
					$this->api_results->events[ $key ]->logo_url = $event->logo->original->url;
				}
			}
		}

		if ( isset( $this->query_vars['ID'] ) ) {
			// find a specific ID
			$ids                       = explode( ',', $this->query_vars['ID'] );
			$this->api_results->events = array_filter(
				$this->api_results->events,
				function ( $e ) use ( $ids ) {
					return in_array( $e->ID, $ids );
				}
			);
		}


		if ( $wfea_fs->is__premium_only() ) {
			if ( isset( $this->query_vars['series_id'] ) ) {
				// find a specific ID
				$ids                       = explode( ',', $this->query_vars['series_id'] );
				$this->api_results->events = array_filter(
					$this->api_results->events,
					function ( $e ) use ( $ids ) {
						if ( property_exists( $e, 'series_id' ) ) {
							return false;
						}

						return in_array( $e->series_id, $ids );
					}
				);
			}
			if ( isset( $this->query_vars['link_custom_page'] ) ) {
				// replace EB url with custom url
				$url                       = $this->query_vars['link_custom_page'];
				$this->api_results->events = array_filter(
					$this->api_results->events,
					function ( $e ) use ( $url ) {
						$e->url = strtr(
							$url,
							array(
								'{id}'    => (int) $e->ID,
								'{title}' => esc_attr(
									strtolower(
										preg_replace(
											'/\\s+/',
											"-",
											trim(
												preg_replace(
													'/[^A-Za-z0-9]/',
													" ",
													$e->post_title )
											)
										)
									)
								),
							)
						);

						return $e;
					}
				);
			}
		}

		// filter out live events that have actually past their end time
		if ( isset( $this->query_vars['status'] ) && 'live' == $this->query_vars['status'] ) {
			$this->api_results->events = array_filter(
				$this->api_results->events,
				function ( $e ) {
					$epoch_end = strtotime( $e->end->utc );
					$result    = ( $epoch_end > time() );

					return $result;
				}
			);
		}

		// now sort  as last step
		$this->sort_results();

		// Limit the number of results: 'limit' - must be last after sort
		if ( isset( $this->query_vars['limit'] ) && is_integer( $this->query_vars['limit'] ) ) {
			$this->api_results->events = array_slice( $this->api_results->events, 0, absint( $this->query_vars['limit'] ) );
		}

	}

	/**
	 * @return bool|false|int
	 */
	protected function get_descriptions__premium_only() {

		//$this->api_results->events;
		$token = false;
		if ( isset( $this->query_vars['token'] ) && ! empty( $this->query_vars['token'] ) ) {
			$token = $this->query_vars['token'];
		}
		foreach ( $this->api_results->events as $key => $event ) {
			$results = Eventbrite_Manager::$instance->get_descriptions( $event->ID, $token );
			if ( isset( $results->description ) ) {
				//	$this->api_results->events[ $key ]->post_content = preg_replace( '/^.+\n/', '', $results->description );
				$this->api_results->events[ $key ]->post_content     = force_balance_tags( $results->description );
				$this->api_results->events[ $key ]->long_description = force_balance_tags( $results->description );
			} else {
				$this->api_results->events[ $key ]->long_description = $this->api_results->events[ $key ]->post_content;
			}
		}

	}

	protected function get_performances__premium_only() {

		/** @TODO
		 *
		 * need to async this
		 */

		$token = false;
		if ( isset( $this->query_vars['token'] ) && ! empty( $this->query_vars['token'] ) ) {
			$token = $this->query_vars['token'];
		}
		foreach ( $this->api_results->events as $key => $event ) {
			if ( '103' == $event->category->id ) {  // 103 is music
				$results                                         = Eventbrite_Manager::$instance->get_performances__premium_only( $event->ID, $token );
				$this->api_results->events[ $key ]->performances = $results->performances;
			}
		}

	}

	protected function add_address_to_events__premium_only() {
		$venue = (object) array(
			'address'         =>
				(object) array(
					'address_1'                            => '',
					'address_2'                            => '',
					'city'                                 => '',
					'region'                               => '',
					'postal_code'                          => '',
					'country'                              => '',
					'latitude'                             => '',
					'longitude'                            => '',
					'localized_address_display'            => '',
					'localized_area_display'               => '',
					'localized_multi_line_address_display' =>
						array(),
				),
			'resource_uri'    => '',
			'id'              => '',
			'age_restriction' => null,
			'capacity'        => null,
			'name'            => '',
			'latitude'        => '',
			'longitude'       => '',
		);

		$fields = explode( ',', $this->query_vars['online_events_address_text'] );
		if ( 0 === count( $fields ) ) {
			return;
		}
		foreach ( $fields as $field ) {
			$atts = explode( ':', $field );
			if ( 2 !== count( $atts ) ) {
				continue;
			}
			if ( property_exists( $venue->address, trim( $atts[0] ) ) ) {
				$venue->address->{trim( $atts[0] )} = trim( $atts[1] );
			}
			if ( property_exists( $venue, trim( $atts[0] ) ) ) {
				$venue->{trim( $atts[0] )} = trim( $atts[1] );
			}

		}

		$this->api_results->events = array_map(
			function ( $event ) use ( $venue ) {
				if ( ! $event->online_event || ! empty( $event->venue ) ) {
					return $event;
				}
				$event->venue = $venue;

				return $event;
			},
			$this->api_results->events
		);


	}

	private function sort_results() {
		if ( ! isset( $this->query_vars['order_by'] ) ) {
			$order_func = 'start_asc';
		} else {
			$order_func = $this->query_vars['order_by'];
		}
		usort( $this->api_results->events, array( $this, $order_func ) );
	}

	/**
	 * Set properties based on the fully processed results.
	 *
	 * @access protected
	 */
	protected function set_properties() {
		/**
		 * @var \Freemius $wfea_fs Object for freemius.
		 */
		global $wfea_fs;
		if ( empty( $this->api_results->events ) ) {
			$this->posts = array();
		} else {
			// Set found_posts based on all posts returned after Eventbrite_Query filtering.
			$this->found_posts = count( $this->api_results->events );

			// Return all posts if `nopaging` is true.
			if ( isset( $this->query_vars['nopaging'] ) && true === $this->query_vars['nopaging'] ) {
				$this->posts    = array_slice( $this->api_results->events, 0, $this->found_posts );
				$posts_per_page = count( $this->posts );
			} else {
				if ( $wfea_fs->is__premium_only() ) {
					// Determine posts according to any pagination querying. Math hurts.
					$modulus        = ( 2 <= $this->query_vars['paged'] && 0 == $this->query_vars['paged'] % 5 ) ? 5 : $this->query_vars['paged'] % 5;
					$offset         = ( 2 <= $modulus && 5 >= $modulus ) ? ( $modulus - 1 ) * $this->query_vars['events_per_page'] : 0;
					$this->posts    = array_slice( $this->api_results->events, $offset, $this->query_vars['events_per_page'] );
					$posts_per_page = $this->query_vars['events_per_page'];
				}
			}

			// Turn the posts into Eventbrite_Event objects.
			$this->posts = array_map( array( $this, 'create_eventbrite_event' ), $this->posts );

			// Determine the maximum number of pages.
			$this->post_count    = count( $this->posts );
			$this->max_num_pages = ceil( $this->found_posts / $posts_per_page );

			// Set the first post.
			$this->post = reset( $this->posts );
		}

		// Adjust some WP_Query parsing.
		if ( ! empty( $this->query_vars['p'] ) ) {
			$this->is_single = true;
		} else {
			$this->is_category = true;
			$this->is_archive  = true;
			$this->is_page     = false;
		}
		$this->is_home = false;
	}

	/**
	 * Turn a given event into a proper Eventbrite_Event object.
	 *
	 * @access protected
	 *
	 * @param null|object $event An event object from the API results.
	 *
	 * @return object Eventbrite_Event object.
	 */
	protected function create_eventbrite_event( $event = null ) {
		// Bail if nothing is passed in.
		if ( empty( $event ) ) {
			return null;
		}

		if ( is_a( $event, 'Eventbrite_Event' ) ) {
			// We already have an Eventbrite_Event object. Nothing to do here.
			$_event = $event;
		} elseif ( is_object( $event ) ) {
			// Looks like we have an object already, so make it an Eventbrite_Event object.
			$_event = new Eventbrite_Event( $event );
		} else {
			return null;  // used to be ID
		}

		// That was a bust. We've got nothing.
		if ( ! $_event ) {
			return null;
		}

		// Return our Eventbrite_Event object.
		return $_event;
	}

	/**
	 * Determine by ID if an event is to be filtered out.
	 *
	 * @access protected
	 *
	 * @param object $event A single event from the API call results.
	 *
	 * @return bool True with no ID match, false if the ID is in the array of events to be removed.
	 */
	protected function filter_by_post_not_in( $event ) {
		// Allow events not found in the array.
		return ! in_array( $event->ID, $this->query_vars['post__not_in'] );
	}

	/**
	 * Filter by title.
	 *
	 * @access protected
	 *
	 * @param object $event A single event from the API call results.
	 *
	 * @return bool True if properties match, false otherwise.
	 */
	protected function filter_by_title__premium_only( $event ) {

		return ( isset( $event->post_title ) ) ? preg_match( $this->query_vars['filter_title'], $event->post_title ) : false;
	}

	/**
	 * Filter by location.
	 *
	 * @access protected
	 *
	 * @param object $event A single event from the API call results.
	 *
	 * @return bool True if properties match, false otherwise.
	 */
	protected function filter_by_location__premium_only( $event ) {
		if ( isset( $event->venue->name ) ) {
			if ( preg_match( $this->query_vars['filter_location'], $event->venue->name ) ) {
				return true;
			}
		}
		if ( isset( $event->venue->address->address_1 ) ) {
			if ( preg_match( $this->query_vars['filter_location'], $event->venue->address->address_1 ) ) {
				return true;
			}
		}
		if ( isset( $event->venue->address->address_2 ) ) {
			if ( preg_match( $this->query_vars['filter_location'], $event->venue->address->address_2 ) ) {
				return true;
			}
		}
		if ( isset( $event->venue->address->city ) ) {
			if ( preg_match( $this->query_vars['filter_location'], $event->venue->address->city ) ) {
				return true;
			}
		}
		if ( isset( $event->venue->address->region ) ) {
			if ( preg_match( $this->query_vars['filter_location'], $event->venue->address->region ) ) {
				return true;
			}
		}
		if ( isset( $event->venue->address->postal_code ) ) {
			if ( preg_match( $this->query_vars['filter_location'], $event->venue->address->postal_code ) ) {
				return true;
			}
		}
		if ( isset( $event->venue->address->country ) ) {
			if ( preg_match( $this->query_vars['filter_location'], $event->venue->address->country ) ) {
				return true;
			}
		}
		if ( isset( $event->venue->address->localized_address_display ) ) {
			if ( preg_match( $this->query_vars['filter_location'], $event->venue->address->localized_address_display ) ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Determine if an event is private.
	 *
	 * @access protected
	 *
	 * @param object $event A single event from the API call results.
	 *
	 * @return bool True if properties match, false otherwise.
	 */
	protected function filter_by_display_private( $event ) {
		return ( isset( $event->public ) ) ? ( $event->public != ( $this->query_vars['display_private'] ) || ( $this->query_vars['display_private'] ) ) : false;
	}

	/**
	 * Determine if an event is managed by a certain organizer.
	 *
	 * @access protected
	 *
	 * @param object $event A single event from the API call results.
	 *
	 * @return bool True if properties match, false otherwise.
	 */
	protected function filter_by_organizer__premium_only( $event ) {
		return ( isset( $event->organizer->id ) ) ? $event->organizer->id == $this->query_vars['organizer_id'] : false;
	}

	/**
	 * Determine if an event is occurring at a given venue.
	 *
	 * @access protected
	 *
	 * @param object $event A single event from the API call results.
	 *
	 * @return bool True if properties match, false otherwise.
	 */
	protected function filter_by_venue__premium_only( $event ) {
		$venues = explode( ',', $this->query_vars['venue_id'] );

		return ( isset( $event->venue->id ) ) ? in_array( $event->venue->id, $venues ) : false;
	}

	/**
	 * Determine if an event is part of a given category.
	 *
	 * @access protected
	 *
	 * @param object $event A single event from the API call results.
	 *
	 * @return bool True if properties match, false otherwise.
	 */
	protected function filter_by_category__premium_only( $event ) {
		$cats = explode( ',', $this->query_vars['category_id'] );

		return ( isset( $event->category->id ) ) ? in_array( $event->category->id, $cats ) : false;
	}

	/**
	 * Determine if an event is part of a given subcategory.
	 *
	 * @access protected
	 *
	 * @param object $event A single event from the API call results.
	 *
	 * @return bool True if properties match, false otherwise.
	 */
	protected function filter_by_subcategory__premium_only( $event ) {
		$sub_cats = explode( ',', $this->query_vars['subcategory_id'] );

		return ( isset( $event->subcategory->id ) ) ? in_array( $event->subcategory->id, $sub_cats ) : false;
	}

	/**
	 * Determine if an event online only.
	 *
	 * @access protected
	 *
	 * @param object $event A single event from the API call results.
	 *
	 * @return bool True if properties match, false otherwise.
	 */
	protected function filter_online_only__premium_only( $event ) {
		return ( isset( $event->online_event ) ) ? $event->online_event : false;
	}

	/**
	 * Determine if an event is part of a given format.
	 *
	 * @access protected
	 *
	 * @param object $event A single event from the API call results.
	 *
	 * @return bool True if properties match, false otherwise.
	 */
	protected function filter_by_format__premium_only( $event ) {
		$formats = explode( ',', $this->query_vars['format_id'] );

		return ( isset( $event->format->id ) ) ? in_array( $event->format->id, $formats ) : false;
	}

	/**
	 * Determine if an event matches search string.
	 *
	 * @access protected
	 *
	 * @param object $event A single event from the API call results.
	 *
	 * @return bool True if properties match, false otherwise.
	 */
	protected function search__premium_only( $event ) {
		// build regex

		if ( empty( $event->venue ) ) {
			$venue = '';
		} else {
			$venue = $event->venue->name;
		}
		$haystack = $event->post_title . ' ' . $venue . ' ' . $event->post_content;

		// always return true if the search has an empty string
		if ( empty( $this->query_vars['search'] ) ) {
			return true;
		}

		if ( empty ( $haystack ) ) {
			return false;
		}

		return preg_match( $this->query_vars['search'], $haystack );
	}

	private function start_asc( $a, $b ) {
		return strcmp( $a->post_date_gmt, $b->post_date_gmt );
	}

	private function start_desc( $a, $b ) {
		return strcmp( $b->post_date_gmt, $a->post_date_gmt );
	}

	private function created_asc( $a, $b ) {
		return strcmp( $a->created, $b->created );
	}

	private function created_desc( $a, $b ) {
		return strcmp( $b->created, $a->created );
	}

	private function published_asc( $a, $b ) {
		return strcmp( $a->eb_published, $b->eb_published );
	}

	private function published_desc( $a, $b ) {
		return strcmp( $b->eb_published, $a->eb_published );
	}


}
